/**
 * @file
 * ToC display.
 */
(function ($, Drupal) {
  "use strict";
  Drupal.skillingSettings.TocInitialized = false;
  Drupal.behaviors.completionScoreDisplay = {
    attach: function (context, settings) {
      if ( Drupal.skillingSettings.TocInitialized ) {
        return;
      }
      Drupal.skillingSettings.TocInitialized = true;
      $(document).ready(function () {
        // Find all of the headings.
        let headingsData = findHeadings();
        // Exit if there are none.
        if (headingsData.length === 0) {
          return;
        }
        // Generate HTML based on the headings array.
        // HTML with links.
        let htmlWithLinks = makeHtmlForHeadingList(headingsData, true);
        // HTML without links.
        let htmlWithoutLinks = makeHtmlForHeadingList(headingsData, false);
        // Add HTML to document, where the ToC Drupal plugin has added markers.
        insertTocHtmlInDocument(htmlWithLinks, htmlWithoutLinks);
      });

      /**
       * Make an array of heading data for the page.
       *
       * @returns {Array}
       *   Array of objects for each heading.
       */
      let findHeadings = function () {
        let headings = [];
        // Find the content area.
        let contentArea = $("div.node__content");
        // Find all the heading tags in the content area from h2 onward.
        let headingTags = contentArea.find("h2, h3, h4, h5, h6");
        // Exit if there are none.
        if (headingTags.length === 0) {
          return headings;
        }
        // For each tag...
        headingTags.each(function(index, element) {
          let headingDeets = extractHeadingTagDeets(element);
          // Add to result.
          headings.push(headingDeets);
        });
        // Return the array.
        return headings;
      };

      let headingTagCount = 0;
      let extractHeadingTagDeets = function (headingTag) {
        // Find the id for the tag, to use in the jump.
        let headingTagId;
        let $headingTag = $(headingTag);
        // Does the heading have an id?
        if ($headingTag.attr("id")) {
          // Yes. Use that.
          headingTagId = $headingTag.attr("id");
        }
        else {
          // Make an id.
          headingTagId = "skilling-toc-" + headingTagCount;
          headingTagCount++;
        }
        // Make an indentation class name based on the tag.
        let tag = $headingTag.get(0).tagName;
        let tagLevel = tag.substr(1);
        let indentationClass = "skilling-toc-heading" + tagLevel;
        // Get the text of the heading.
        let text = $headingTag.text();
        return {
          id: headingTagId,
          text: text,
          class: indentationClass
        };
      };

      let makeHtmlForHeadingList = function (headingTagsDeets, includeLinks) {
        // What class will the list items have?
        let linkTagClass = includeLinks
            ? "skilling-toc-with-links"
            : "skilling-toc-without-links";
        // Create the list opening tag.
        let result = "<ul class='" + linkTagClass + "'>\n";
        // Loop across the array.
        for (let listItem of headingTagsDeets) {
          let listItemTag = "<li class='" + listItem.class + "'>";
          if (includeLinks) {
            listItemTag += "<a href='#" + listItem.id + "'>" + listItem.text + "</a>";
          }
          else {
            listItemTag += listItem.text;
          }
          listItemTag += "</li>";
          // Add list item tag to overall result.
          result += listItemTag;
        }
        result += "</ul>";
        return result;
      };

      let insertTocHtmlInDocument = function(htmlWithLinks, htmlWithoutLinks) {
        // Find the insertion locations.
        $("div.skilling-toc-container").each(function(index, tocMarker){
          let $tocMarker = $(tocMarker);
          let showLinks = $tocMarker.data("links");
          if (showLinks) {
            $tocMarker.html(htmlWithLinks);
          }
          else {
            $tocMarker.html(htmlWithoutLinks);
          }
        });
      };


    }
  }
})(jQuery, Drupal);
