<?php

namespace Drupal\skilling\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Watch events.
 */
class SkillingSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = ['routingRouteFinished'];
    return $events;
  }

  /**
   * This method is called when the routing.route_finished event is dispatched.
   *
   * @param \Symfony\Component\EventDispatcher\Event $event
   *   What happened.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\skilling\Exception\SkillingException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function routingRouteFinished(GetResponseEvent $event) {
    /** @var \Drupal\skilling\BookUpdateTracker $bookUpdateTracker */
    $bookUpdateTracker = \Drupal::service('skilling.book_update_tracker');
    $bookUpdateTracker->runScheduledBookOrderUpdates();
    /** @var \Drupal\skilling\RecomputeClassRoles $recomputeClassRoles */
    $recomputeClassRoles = \Drupal::service('skilling.recompute_class_roles');
    $recomputeClassRoles->runScheduledClassRoleUpdates();
    // Recompute completion score if needed.
    /** @var \Drupal\skilling\SkillingCurrentUser $currentUser */
    $currentUser = \Drupal::service('skilling.skilling_current_user');
    if ($currentUser->isStudent()) {
      /** @var \Drupal\skilling\CurrentClass $currentClass */
      $currentClass = \Drupal::service('skilling.current_class');
      if ($currentClass) {
        /** @var \Drupal\skilling\CompletionScore $completionScoreService */
        $completionScoreService = \Drupal::service('skilling.completion_score');
        $completionScoreService->getCompletionScoreForCurrentEnrollment();
      }
    }
  }

}
