<?php

namespace Drupal\skilling;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\node\NodeInterface;
use Drupal\skilling\Access\FilterUserInput;
use Drupal\skilling\Access\SkillingAjaxSecurityInterface;
use Drupal\skilling\Exception\SkillingUnknownValueException;
use Drupal\skilling\Exception\SkillingWrongTypeException;
use Drupal\skilling\Plugin\HistoryItemManager;

/**
 * Make a history event object for use with a history node.
 *
 * The object is a plugin that wraps the node. It includes methods
 * that can create renderable and XML representations of the event deets.
 *
 * @package Drupal\skilling
 */
class HistoryItemFactory {

  use StringTranslationTrait;

  /**
   * Plugin item manager service.
   *
   * @var \Drupal\skilling\Plugin\HistoryItemManager
   */
  protected $historyItemManager;

  /**
   * Current user service.
   *
   * Factories that make HistoryItems have to set this.
   *
   * @var \Drupal\skilling\SkillingCurrentUser
   */
  protected $skillingCurrentUser;

  /**
   * Service to check AJAX calls.
   *
   * @var \Drupal\skilling\Access\SkillingAjaxSecurityInterface
   */
  protected $ajaxSecurityService;

  /**
   * Service to filter user input.
   *
   * @var \Drupal\skilling\Access\FilterUserInputInterface
   */
  protected $filterInputService;

  /**
   * HistoryItemFactory constructor.
   *
   * @param \Drupal\skilling\Plugin\HistoryItemManager $historyItemManager
   *   Plugin item manager service.
   * @param \Drupal\skilling\SkillingCurrentUser $skillingCurrentUser
   *   Current user service.
   * @param \Drupal\skilling\Access\SkillingAjaxSecurityInterface $ajaxSecurityService
   * @param \Drupal\skilling\Access\FilterUserInput $filterInputService
   */
  public function __construct(
    HistoryItemManager $historyItemManager,
    SkillingCurrentUser $skillingCurrentUser,
    SkillingAjaxSecurityInterface $ajaxSecurityService,
    FilterUserInput $filterInputService
  ) {
    $this->historyItemManager = $historyItemManager;
    $this->skillingCurrentUser = $skillingCurrentUser;
    $this->ajaxSecurityService = $ajaxSecurityService;
    $this->filterInputService = $filterInputService;
  }

  /**
   * Make a history event object for use with a history node.
   *
   * @param \Drupal\node\NodeInterface $node
   *   A history node.
   *
   * @return \Drupal\skilling\Plugin\HistoryItemInterface
   *   The history plugin instance.
   *
   * @throws \Drupal\skilling\Exception\SkillingUnknownValueException
   * @throws \Drupal\skilling\Exception\SkillingWrongTypeException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function makeHistoryEventObject(NodeInterface $node) {

    return NULL;

    // Is the param a history node?
    if ($node->bundle() !== SkillingConstants::HISTORY_CONTENT_TYPE) {
      throw new SkillingWrongTypeException(
        $this->t("For history nodes only."), __FILE__, __LINE__
      );
    }
    // Get the name of the event type.
    $eventType = $node->get('field_event_type_name')->value;
    $plugin = $this->makeNodelessHistoryEventObject($eventType);
    // Remember the node being wrapped.
    $plugin->setHistoryEventNode($node);
    return $plugin;
  }

  /**
   * Make a history event object of a given type.
   *
   * @param string $eventType
   *   Event type, e.g., view_lesson.
   *
   * @return \Drupal\skilling\Plugin\HistoryItemInterface
   *   The history plugin instance.
   *
   * @throws \Drupal\skilling\Exception\SkillingUnknownValueException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function makeNodelessHistoryEventObject($eventType) {

    return NULL;

    $historyPluginDefinitions = $this->historyItemManager->getDefinitions();
    // Find plugin id.
    $eventPluginId = FALSE;
    foreach ($historyPluginDefinitions as $pluginId => $historyPluginDefinition) {
      if ($historyPluginDefinition['event_type_name'] == $eventType) {
        $eventPluginId = $pluginId;
        break;
      }
    }
    if (!$eventPluginId) {
      throw new SkillingUnknownValueException(
        $this->t('Unknown event type'), $eventType, __FILE__, __LINE__
      );
    }
    /* @var \Drupal\skilling\Plugin\HistoryItemInterface $plugin */
    $plugin = $this->historyItemManager->createInstance($eventPluginId);
    $plugin->setCurrentUser($this->skillingCurrentUser);
    $plugin->setAjaxSecurityService($this->ajaxSecurityService);
    $plugin->setFilterUserInputService($this->filterInputService);
    return $plugin;
  }

}
