<?php

namespace Drupal\skilling\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\Renderer;
use Drupal\skilling\Timeline;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a timeline block.
 *
 * @Block(
 *  id = "timeline_block",
 *  admin_label = @Translation("Timeline block"),
 * )
 */
class TimelineBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Renderer service.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * Timeline service.
   *
   * @var \Drupal\skilling\Timeline
   */
  protected $timelineService;

  /**
   * Constructs a new CalendarBlock object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param string $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Render\Renderer $renderer
   *   Renderer service.
   * @param \Drupal\skilling\Timeline $timelineService
   *   Timeline service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, Renderer $renderer, Timeline $timelineService) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->renderer = $renderer;
    $this->timelineService = $timelineService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    /* @noinspection PhpParamsInspection */
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('renderer'),
      $container->get('skilling.timeline')
    );
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Exception
   */
  public function build() {
    list($problemMessage, $weeksForTemplate) = $this->timelineService->makeTimeline();
    $renderable = [
      '#theme' => 'timeline',
      '#problem_message' => $problemMessage,
      '#weeks' => $weeksForTemplate,
      '#attached' => [
        'library' => ['skilling/timeline'],
      ],
      '#cache' => ['max-age' => 0],
    ];
    return $renderable;
  }

}
