<?php

namespace Drupal\skilling\Plugin\HistoryItem;

use Drupal\Component\Utility\Html;
use Drupal\skilling\Plugin\HistoryItemBase;

/**
 * Provides a complete FiB history event.
 *
 * @SkillingHistoryItem(
 *  id = "skilling_complete_fib_history_event",
 *  event_type_name = "complete_fib",
 *  label = @Translation("Complete FiB"),
 *  description = @Translation("User completes an FiB."),
 * )
 */
class CompleteFib extends HistoryItemBase {

  /**
   * {@inheritdoc}
   */
  public function getDetailsRenderable() {
    list($fibNid, $internalName, $response) = $this->extractDeets();
    $result = [
      '#theme' => 'history_fib_response_deets',
      '#fib_nid' => $fibNid,
      '#internal_name' => $internalName,
      '#response' => Html::escape($response),
    ];
    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function getDetailsXml() {
    list($fibNid, $internalName, $response) = $this->prepareForXml($this->extractDeets());
    $result =
      '<fib-response nid="' . $fibNid . '" internal-name="' . $internalName . '">'
        . $response
      . '</fib-response>';
    return $result;
  }

  /**
   * Extract the details into separate variables.
   *
   * @return array
   *   Deets in variables.
   */
  protected function extractDeets() {
    $deets = $this->historyEventNode->get('field_details')->value;
    // Unserialize it.
    $deets = unserialize($deets);
    $fibNid = $deets['fibNid'];
    $internalName = $deets['internalName'];
    $response = $this->filterInputService->filterUserContent($deets['response']);
    return [$fibNid, $internalName, $response];
  }

}
