<?php

namespace Drupal\skilling\Plugin\HistoryItem;

use Drupal\skilling\Plugin\HistoryItemBase;

/**
 * Provides a complete MCQ history event.
 *
 * @SkillingHistoryItem(
 *  id = "skilling_complete_mcq_history_event",
 *  event_type_name = "complete_mcq",
 *  label = @Translation("Complete MCQ"),
 *  description = @Translation("User completes an MCQ."),
 * )
 */
class CompleteMcq extends HistoryItemBase {

  /**
   * {@inheritdoc}
   */
  public function getDetailsRenderable() {
    list($mcqNid, $internalName, $response) = $this->extractDeets();
    $result = [
      '#theme' => 'history_mcq_response_deets',
      '#mcq_nid' => $mcqNid,
      '#internal_name' => $internalName,
      '#response' => $response,
    ];
    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function getDetailsXml() {
    list($mcqNid, $internalName, $response) = $this->prepareForXml($this->extractDeets());
    $result = '<mcq-response nid="' . $mcqNid . '" internal-name="' . $internalName . '">'
      . $response . '</mcq-response>';
    return $result;
  }

  /**
   * Extract the details into separate variables.
   *
   * @return array
   *   Deets in variables.
   */
  protected function extractDeets() {
    $deets = $this->historyEventNode->get('field_details')->value;
    // Unserialize it.
    $deets = unserialize($deets);
    $mcqNid = $deets['mcqNid'];
    $internalName = $deets['internalName'];
    $response = $deets['response'];
    return [$mcqNid, $internalName, $response];
  }

}
