<?php

namespace Drupal\skilling\Plugin\HistoryItem;

use Drupal\skilling\Plugin\HistoryItemBase;

/**
 * Provides a submission history event.
 *
 * @SkillingHistoryItem(
 *  id = "skilling_create_exercise_submission_history_event",
 *  event_type_name = "create_exercise_submission",
 *  label = @Translation("Exercise submission created"),
 *  description = @Translation("Student submits an exercise solution."),
 * )
 */
class CreateExerciseSubmission extends HistoryItemBase {

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\skilling\Exception\SkillingException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getDetailsRenderable() {
    $submissionNid = $this->historyEventNode->field_subject_node->target_id;
    list($studentUid, $exerciseNid) = $this->extractDeets();
    $result = [
      '#theme' => 'history_created_submission_deets',
    ];
    // Look up exercise title.
    /** @var \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager */
    $entityTypeManager = \Drupal::service('entity_type.manager');
    $exercise = $entityTypeManager->getStorage('node')
      ->load($exerciseNid);
    if ($exercise) {
      $result['#exerciseNid'] = $exerciseNid;
      $result['#exerciseTitle'] = $exercise->get('title');
    }
    /** @var \Drupal\skilling\SkillingCurrentUser $currentUser */
    $currentUser = \Drupal::service('skilling.skilling_current_user');
    if ($currentUser->isAdministrator()) {
      // Admins get more stuff in their deets.
      /** @var \Drupal\skilling\SkillingUserFactory $userFactory */
      $userFactory = \Drupal::service('skilling.skilling_user_factory');
      $student = $userFactory->makeSkillingUser($studentUid);
      $result['#studentUid'] = $studentUid;
      $result['#studentName'] = $student->getAccountName();
      $result['#submissionNid'] = $submissionNid;
    }
    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function getDetailsXml() {
    list($studentUid, $exerciseNid) = $this->prepareForXml($this->extractDeets());
    $result = '<submission-created '
      . 'student="' . $studentUid . '"'
      . 'exercise="' . $exerciseNid . '"'
      . '>';
    $result .= '</submission-created>';
    return $result;
  }

  /**
   * Extract the details into separate variables.
   *
   * @return array
   *   Deets in variables.
   */
  protected function extractDeets() {
    $deets = $this->historyEventNode->get('field_details')->value;
    // Unserialize it.
    $deets = unserialize($deets);
    $studentUid = $deets['student'];
    $exerciseNid = $deets['exercise'];
    return [
      $studentUid,
      $exerciseNid,
    ];
  }

  /**
   * Is the event to be logged in history?
   *
   * Always log.
   *
   * @return bool
   *   Whether to log the event.
   */
  public function isLogged() {
    return TRUE;
  }

}
