<?php

namespace Drupal\skilling\Plugin;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;
use Drupal\skilling\Access\FilterUserInputInterface;
use Drupal\skilling\Access\SkillingAjaxSecurityInterface;
use Drupal\skilling\SkillingCurrentUser;

/**
 * Defines an interface for Skilling history item plugins.
 */
interface HistoryItemInterface extends PluginInspectionInterface {

  /**
   * Set the node being wrapped.
   *
   * @param \Drupal\node\NodeInterface $historyEventNode
   *   The node.
   */
  public function setHistoryEventNode(NodeInterface $historyEventNode);

  /**
   * Get the node being wrapped.
   *
   * @return \Drupal\node\NodeInterface
   *   The node.
   */
  public function getHistoryEventNode();

  /**
   * Get a renderable array for showing deets of a history event.
   *
   * @return array
   *   The renderable.
   */
  public function getDetailsRenderable();

  /**
   * Get XML representation of the deets of a history event.
   *
   * @return string
   *   The XML.
   */
  public function getDetailsXml();

  /**
   * Is the event to be logged in history?
   *
   * @return bool
   *   Whether to log the event.
   */
  public function isLogged();

  /**
   * Get the uid the event is for.
   *
   * Is almost always the uid of the current user. Plugin implementations
   * can do something different if they want.
   *
   * @return int
   *   The uid.
   */
  public function getUidEventIsFor();

  /**
   * Set the current user.
   *
   * @param \Drupal\skilling\SkillingCurrentUser $skillingCurrentUser
   *   The logged in user.
   */
  public function setCurrentUser(SkillingCurrentUser $skillingCurrentUser);

  /**
   * Set the AJAX security service.
   *
   * @param \Drupal\skilling\Access\SkillingAjaxSecurityInterface $ajaxSecurityService
   *   The service.
   */
  public function setAjaxSecurityService(SkillingAjaxSecurityInterface $ajaxSecurityService);

  /**
   * Set the user input filter service.
   *
   * @param \Drupal\skilling\Access\FilterUserInputInterface $filterInputService
   *   The service.
   */
  public function setFilterUserInputService(FilterUserInputInterface $filterInputService);

  /**
   * Report an error. Wrap a message with Useful Stuff.
   *
   * @param string $message
   *   Error message to be reported.
   *
   * @return mixed
   *   Complete error report.
   */
  public function formatHistoryItemError($message);

}
