<?php

namespace Drupal\skilling\Plugin\views\field;

use Drupal\skilling\SkillingConstants;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;

/**
 * Provide a field showing the details of a history event.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("history_details_display_field")
 */
class HistoryDetailsDisplayField extends FieldPluginBase {

  /**
   * {@inheritdoc}
   */
  public function usesGroupBy() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    // Do nothing -- to override the parent query.
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();

    $options['hide_alter_empty'] = ['default' => FALSE];
    return $options;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\skilling\Exception\SkillingValueMissingException
   * @throws \Drupal\skilling\Exception\SkillingWrongTypeException
   * @throws \Drupal\skilling\Exception\SkillingException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function render(ResultRow $values) {
    $node = $values->_entity;
    if ($node->bundle() !== SkillingConstants::HISTORY_CONTENT_TYPE) {
      return ($this->t("For history nodes only."));
    }
    /** @var \Drupal\skilling\HistoryItemFactory $factory */
    $factory = \Drupal::service('skilling.history_item_factory');
    $historyEvent = $factory->makeHistoryEventObject($node);
    $display = $historyEvent->getDetailsRenderable();
    return $display;
  }

}
